"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const chalk_1 = __importDefault(require("chalk"));
const child_process_1 = require("child_process");
const inquirer_1 = require("inquirer");
const api_1 = require("./api");
const assertNever_1 = require("./assertNever");
const cli = __importStar(require("./cli"));
const printer_1 = require("./printer");
const sleep_1 = require("./sleep");
function sortByDesc(array, fn) {
    return [...array].sort((a, b) => {
        const xa = fn(a);
        const xb = fn(b);
        if (xa < xb) {
            return 1;
        }
        else if (xa > xb) {
            return -1;
        }
        else {
            return 0;
        }
    });
}
function fallbackArray(array, fallback) {
    if (array.length === 0) {
        return fallback;
    }
    return array;
}
function getPagerStream() {
    const pager = process.platform === "win32"
        ? child_process_1.spawn("more", { stdio: ["pipe", "inherit", "inherit"] })
        : child_process_1.spawn("less", ["-R"], { stdio: ["pipe", "inherit", "inherit"] });
    if (pager.stdin) {
        // I don't care if the user quits their pager before we've written all our
        // output! Just silently ignore so the program can finish.
        pager.stdin.on("error", (err) => {
            if (err.code !== "EPIPE") {
                throw err;
            }
        });
        return pager.stdin;
    }
    return process.stdout;
}
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        // Validate the `--report` command line option
        if (cli.report && !["usage", "props", "lines"].includes(cli.report)) {
            throw new Error(`jsx-info: invalid report: ${cli.report} (expected: usage | props | lines)`);
        }
        // Interactively ask the user for any missing command line flags that are
        // required
        const answers = yield inquirer_1.prompt([
            {
                type: "input",
                name: "components",
                when: !cli.components,
                default: "*",
                filter(input) {
                    input = input.trim();
                    if (input === "") {
                        return ["*"];
                    }
                    return input.split(/\s+/);
                },
                message: "Which components (space separated, * = all) [--components]",
            },
            {
                type: "list",
                name: "report",
                when: !cli.report,
                message: "Which report [--report]",
                choices: [
                    { value: "usage", name: "Total component usage [usage]" },
                    { value: "props", name: "Total props usage [props]" },
                    {
                        value: "lines",
                        name: "Show lines where certain props are used [lines]",
                    },
                ],
            },
            {
                type: "input",
                name: "prop",
                when({ report }) {
                    return (report === "lines" || cli.report === "lines") && !cli.prop;
                },
                validate(input) {
                    return input.trim() !== "";
                },
                filter(input) {
                    return input.trim();
                },
                message: "Which prop (e.g. `id` or `variant=primary` or `!className` or `type!=text`) [--prop]",
            },
        ]);
        if (Object.keys(answers).length > 0) {
            printer_1.print();
        }
        // The internal representation for "all components" is different than how you
        // actually pass it in via the command line, since it's weird to pass an
        // "empty array" as a command line flag.
        const components = ((comp) => {
            if (comp && comp.length == 1 && comp[0] == "*") {
                return [];
            }
            return comp;
        })(answers.components || cli.components);
        // Merge the command line flags with the interactive questions that were
        // asked, and fall back to default values.
        const prop = answers.prop || cli.prop;
        const report = answers.report || cli.report || "usage";
        const files = fallbackArray(cli.files, ["**/*.{js,jsx,tsx}"]);
        // Start the spinner. Sometimes the "finding files" step can take a long time
        // if you accidentally scan the node_modules folder. This way you can tell
        // what's wrong instead of just seeing the program hang.
        if (cli.showProgress) {
            printer_1.spinner.text = "Finding files";
            printer_1.spinner.start();
        }
        // Start the analysis
        const results = yield api_1.analyze({
            components,
            files,
            prop,
            babelPlugins: cli.babelPlugins,
            directory: cli.directory,
            gitignore: cli.gitignore,
            ignore: cli.ignore,
            // Update the loading spinner with the current filename, and `sleep` so that
            // the loading spinner actually gets a chance to run, instead of us chewing
            // all the CPU time.
            onFile(filename) {
                return __awaiter(this, void 0, void 0, function* () {
                    if (cli.showProgress) {
                        printer_1.spinner.text = `Scanning files\n\n${filename}`;
                        yield sleep_1.sleep();
                    }
                });
            },
        });
        // Stop the spinner now that the analysis is done
        printer_1.spinner.stop();
        // There doesn't seem to be any good way to output color codes into a pager in
        // Windows, so let's briefly disable color output from chalk.
        const oldChalkLevel = chalk_1.default.level;
        if (process.platform === "win32") {
            chalk_1.default.level = 0;
        }
        // Create a stream that writes to a pager (`less -R` unless you're on Windows,
        // then it's `more`).
        const stream = getPagerStream();
        // Log how long the command took
        reportTime(stream, results);
        // Report whatever the user was interested in
        if (report === "usage") {
            reportComponentUsage(stream, results);
        }
        else if (report === "props") {
            reportPropUsage(stream, results);
        }
        else if (report === "lines") {
            reportLinesUsage(stream, results);
        }
        else {
            assertNever_1.assertNever(report);
        }
        // Report what errors happened so the user can fix parse errors
        reportErrors(stream, results);
        // End the stream so that the pager knows the input is finished
        stream.end();
        // Restore the old chalk color level
        chalk_1.default.level = oldChalkLevel;
    });
}
exports.main = main;
function reportTime(stream, { filenames, elapsedTime }) {
    stream.write(`Scanned ${filenames.length} files in ${elapsedTime.toFixed(1)} seconds\n`);
}
function reportComponentUsage(stream, { componentTotal, componentUsageTotal, componentUsage }) {
    if (componentTotal === 0) {
        return;
    }
    const pairs = componentUsage;
    const maxDigits = getMaxDigits(Object.values(pairs));
    stream.write(chalk_1.default.cyan `\n${componentTotal} components used ${componentUsageTotal} times:\n`);
    const sortedUsage = sortByDesc(Object.entries(pairs), ([, count]) => count);
    for (const [componentName, count] of sortedUsage) {
        stream.write([
            "  " + chalk_1.default.bold(count.toString().padStart(maxDigits)),
            "  " + printer_1.textMeter(componentUsageTotal, count),
            "  " + chalk_1.default.bold(`<${componentName}>`) + "\n",
        ].join(""));
    }
}
function reportLinesUsage(stream, { lineUsage }) {
    for (const [componentName, props] of Object.entries(lineUsage)) {
        for (const data of Object.values(props)) {
            for (const lineData of data) {
                const { filename, startLoc, prettyCode } = lineData;
                const { line, column } = startLoc;
                const styledComponentName = chalk_1.default.bold(`<${componentName}>`);
                stream.write(chalk_1.default.cyan `\n${styledComponentName} ${filename}:${line}:${column}\n`);
                stream.write(prettyCode);
                stream.write("\n");
            }
        }
    }
}
function getMaxDigits(iterable) {
    // Pad to at least 4 digits so that most things line up
    return Math.max(4, ...[...iterable].map((n) => n.toString().length));
}
function reportPropUsage(stream, { propUsage, componentUsage }) {
    const propUsageByComponent = sortByDesc(Object.entries(propUsage), ([name]) => componentUsage[name]);
    for (const [componentName, propUsage] of propUsageByComponent) {
        const compUsage = componentUsage[componentName];
        stream.write(`\n${chalk_1.default.bold(`<${componentName}>`)} was used ${compUsage} ${compUsage === 1 ? "time" : "times"} with the following prop usage:\n`);
        const maxDigits = getMaxDigits(Object.values(propUsage));
        const sortedUsage = sortByDesc(Object.entries(propUsage), ([, count]) => count);
        for (const [propName, usage] of sortedUsage) {
            stream.write([
                "  " + chalk_1.default.bold(usage.toString().padStart(maxDigits)),
                "  " + printer_1.textMeter(compUsage, usage),
                "  " + chalk_1.default.bold(propName),
                "\n",
            ].join(""));
        }
    }
}
function reportErrors(stream, { errors, suggestedPlugins }) {
    const errorsCount = Object.keys(errors).length;
    if (errorsCount) {
        stream.write("\n" + errorsCount + " parse " + (errorsCount === 1 ? "error" : "errors"));
        for (const [filename, error] of Object.entries(errors)) {
            const { loc, message } = error;
            const { line, column } = loc;
            stream.write(`  ${filename}:${line}:${column}` + chalk_1.default.bold.red(message) + "\n");
        }
        if (suggestedPlugins.length > 0) {
            stream.write("\nTry adding these Babel plugins as arguments:\n");
            stream.write(chalk_1.default.bold.cyan("    --babel-plugins") +
                " " +
                suggestedPlugins.map((s) => chalk_1.default.underline.magenta(s)).join(" ") +
                "\n");
        }
    }
}
